"""
Copyright (C) 2012 DECOIT GmbH <asterisk4ucs@decoit.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License version 3 as
published by the Free Software Foundation

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import univention.management.console.modules
from univention.management.console.protocol.definitions import SUCCESS
from univention.management.console.log import MODULE

import univention.config_registry
import univention.admin.uldap
import univention.admin.config
import univention.admin.modules
univention.admin.modules.update()

import logging

import univention.admin.handlers.users.user
import univention.admin.handlers.asterisk.sipPhone
import univention.admin.handlers.asterisk.mailbox
import univention.admin.handlers.asterisk.server

logfile = "/var/log/univention/asteriskUserPython.log"
logFilename = "/var/log/univention/asteriskUser.log"

class Instance( univention.management.console.modules.Base ):
	logging.basicConfig(filename=logfile,
		#level=logging.INFO,
          level=logging.DEBUG,
		format = "%(asctime)s\t%(levelname)s\t%(message)s",
		datefmt = "%d.%m.%Y %H:%M:%S"
		)	

	def load( self, request ):
		logging.debug('USER: START LOAD METHOD')
		user, mailbox = getUserAndMailbox(self._user_dn)
		if mailbox == "KeinServer" :
			logging.debug('USER : LOAD: KEIN SERVER')
			result = "KeinServer"
			self.finished( request.id, result )
		else: 
			logging.debug('USER:LOAD:SERVER IST DA')
			result = {
				"phones/interval": user["ringdelay"],
				"forwarding/number": user.get("forwarding",""),

				"mailbox/timeout": user["timeout"],
				"mailbox": False,
			}

			if mailbox:
				result.update({
					"mailbox": True,
					"mailbox/password": mailbox["password"],
					"mailbox/email": mailbox["email"],
				})

			request.status = SUCCESS
			logging.debug('USER LOAD SUCCESS')
			self.finished( request.id, result )

	def save( self, request ):
		user, mailbox = getUserAndMailbox(self._user_dn)

		user["ringdelay"] = request.options["phones/interval"]
		user["timeout"] = request.options["mailbox/timeout"]
		if request.options["forwarding/number"]:
			user["forwarding"] = request.options[
					"forwarding/number"]
		else:
			try:
				del user.info["forwarding"]
			except KeyError:
				pass
		user.modify()

		if mailbox:
			mailbox["password"] = request.options[
							"mailbox/password"]
			mailbox["email"] = request.options["mailbox/email"]
			mailbox.modify()

		request.status = SUCCESS
		self.finished( request.id, "success",
			"Speichern war erfolgreich!" )

	def phonesQuery(self, request):
		logging.debug('phoneQuery: START')
		if (request.options.get("dn") and
				request.options.get("position")
					in ["-1", "1"]):
			changePhoneOrder(
					self._user_dn,
					request.options["dn"],
					int(request.options["position"]))

		phones = getPhones(self._user_dn)

		result = []
		for i,phone in enumerate(phones):
			result.append({
				"position": i,
				"dn": phone.dn,
				"name": phone["extension"],
			})

		request.status = SUCCESS
		logging.debug('phoneQuerys: SUCCESS')
		self.finished(request.id, result)

def getCoLo():
	logging.debug('getCoLo: START: CREATE LDAP CONNECTION')
	# create ldap connection
	ucr = univention.config_registry.ConfigRegistry()
	ucr.load()
	ldap_master = ucr.get('ldap/master', '')
	ldap_base = ucr.get('ldap/base', '')
	binddn = ','.join(('cn=admin', ldap_base))
	bindpw = open('/etc/ldap.secret','r').read().strip()
	lo = univention.admin.uldap.access(host=ldap_master, base=ldap_base,
		binddn=binddn, bindpw=bindpw, start_tls=2)

	co = univention.admin.config.config()
        logging.debug('getCoLo: SUCCESS: co=%s, lo=%s', co, lo)
	return co, lo

def getUser(co, lo, dn):
	logging.debug('getUser: START')
	position = univention.admin.uldap.position(lo.base)

	univention.admin.modules.init(lo, position,
		univention.admin.handlers.users.user)

	obj = univention.admin.handlers.users.user.object(co, lo,
		None, dn)
	obj.open()
	logging.debug('getUser: SUCCESS: obj=%s', obj)
	return obj

def getPhone(co, lo, dn):
	logging.debug('getPhone: START')
	obj = univention.admin.handlers.asterisk.sipPhone.object(co, lo,
		None, dn)
	obj.open()
	logging.debug('getPhone: SUCCESS: obj=%s', obj)
	return obj

def getMailbox(co, lo, dn):
	logging.debug('getMailbox: START')
	obj = univention.admin.handlers.asterisk.mailbox.object(co, lo,
		None, dn)
	obj.open()
	logging.debug('getMailbox: SUCCESS: obj=%s', obj)
	return obj

def getPhones(userdn):
	logging.debug('getPhones: START')
	co, lo = getCoLo()

	user = getUser(co, lo, userdn)

	phones = []
	for dn in user.get("phones", []):
		phones.append(getPhone(co, lo, dn))
	logging.debug('getPhones: SUCCESS: phones=%s', phones)
	return phones

def getUserAndMailbox(userdn):
	logging.debug('getUserAndMailbox: START')
	co, lo, pos = getCoLoPos()

	server = univention.admin.modules.get("asterisk/server")
	univention.admin.modules.init(lo, pos, server)
	objs = server.lookup(co, lo, None)

	checkServers = []
	for obj in objs:
			checkServers.append({
				"label": obj["commonName"],
			})
	
	#MODULE.error('User: server: %s' % len(checkServers))
	if len(checkServers) >0 :
		co, lo = getCoLo()

		user = getUser(co, lo, userdn)

		mailbox = user.get("mailbox")
		if mailbox:
			mailbox = getMailbox(co, lo, mailbox)
		logging.debug('getUserAndMailbox: IF SUCCESS: user=%s, mailbox=%s', user, mailbox)
		return user, mailbox
	elif len(checkServers) == 0 :
		MODULE.error('Fehler gefunden!')
		mailbox = "KeinServer"
		user = "KeinServer"
		logging.debug('getUserAndMailbox: ELSE-IF SUCCESS: user=%s, mailbox=%s',user,mailbox)
		return user, mailbox
		

def getCoLoPos():
	logging.debug('getCoLoPos: START')
	co = univention.admin.config.config()

	lo, pos = univention.admin.uldap.getAdminConnection()
	logging.debug('getCoLoPos: SUCCESS: co= %s, lo= %s, pos=%s', co,lo,pos)
	return co, lo, pos

def changePhoneOrder(userdn, phonedn, change):
	logging.debug('getPhoneOrder: START')
	co, lo = getCoLo()

	user = getUser(co, lo, userdn)

	phones = user.get("phones", [])
	i = phones.index(phonedn)
	phones.pop(i)
	if change == -1 and i > 0:
		phones.insert(i-1, phonedn)
	elif change == 1:
		phones.insert(i+1, phonedn)

	user["phones"] = phones
	user.modify()
 	logging.debug('getPhoneOrder: SUCCESS')
